package cn.ibizlab.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;
import lombok.extern.slf4j.Slf4j;

import cn.ibizlab.util.security.SpringContextHolder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.util.Assert;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.sample.domain.IBIZBOOK;
import cn.ibizlab.core.sample.filter.IBIZBOOKSearchContext;
import cn.ibizlab.core.sample.service.IBIZBOOKService;
import cn.ibizlab.core.sample.mapper.IBIZBOOKMapper;
import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.security.AuthenticationUser;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.core.conditions.Wrapper;


/**
 * 实体[图书] 服务对象接口实现
 */
@Slf4j
@Service("IBIZBOOKService")
public class IBIZBOOKServiceBase extends ServiceImpl<IBIZBOOKMapper,IBIZBOOK> implements IBIZBOOKService {


    protected int batchSize = 500;

    public IBIZBOOK get(IBIZBOOK et) {
        IBIZBOOK rt = this.baseMapper.selectEntity(et);
        Assert.notNull(rt,"数据不存在,图书:"+et.getIbizbookid());
        BeanUtils.copyProperties(rt, et);
        return et;
    }
    
    public List<IBIZBOOK> getByEntities(List<IBIZBOOK> entities) {
        return this.baseMapper.selectEntities(entities);
    }


    public IBIZBOOK getDraft(IBIZBOOK et) {
        return et;
    }

    public Integer checkKey(IBIZBOOK et) {
        return ( (!ObjectUtils.isEmpty(et.getIbizbookid())) && this.count(Wrappers.<IBIZBOOK>query().eq("ibizbookid", et.getIbizbookid()))>0 )?1:0;
    }

    @Override
    @Transactional
    public boolean create(IBIZBOOK et) {
        if(!this.retBool(this.baseMapper.insert(et))) 
            return false;
        get(et);
        return true;
    }
    @Transactional
    public boolean createBatch(List<IBIZBOOK> list) {
        this.saveBatch(list, batchSize);
        return true;
    }

    @Transactional
    public boolean update(IBIZBOOK et) {
        UpdateWrapper qw=et.getUpdateWrapper(true);
        qw.eq("ibizbookid", et.getIbizbookid());
        if(!update(et, qw)) {
            return false;
        }
        get(et);
        return true;
    }

    @Transactional
    public boolean updateBatch(List<IBIZBOOK> list) {
        updateBatchById(list, batchSize);
        return true;
    }

    @Transactional
    public boolean save(IBIZBOOK et) {
        if(checkKey(et)>0)
            return getSelf().update(et);
        else
            return getSelf().create(et);
    }

    @Transactional
    public boolean saveBatch(List<IBIZBOOK> list) {
        List<IBIZBOOK> rt=this.getByEntities(list);
        Set<Serializable> keys=new HashSet<>();
        rt.forEach(et->{
            Serializable key = et.getIbizbookid();
            if(!ObjectUtils.isEmpty(key))
                keys.add(key);
        });
        List<IBIZBOOK> _create=new ArrayList<>();
        List<IBIZBOOK> _update=new ArrayList<>();
        list.forEach(et-> {
            Serializable key = et.getIbizbookid();
            if(keys.contains(key))
                _update.add(et);
            else
                _create.add(et);
        });
        List rtList=new ArrayList<>();
        if(_update.size()>0 && (!getSelf().updateBatch(_update)))
            return false;
        if(_create.size()>0 && (!getSelf().createBatch(_create)))
            return false;
        return true;
    }

    @Transactional
    public boolean remove(IBIZBOOK et) {
        String key = et.getIbizbookid();

        if(!remove(new QueryWrapper<IBIZBOOK>()
                .eq("ibizbookid", et.getIbizbookid())
            )) {
            return false;
        }
        return true ;
    }

    @Transactional
    public boolean removeByEntities(List<IBIZBOOK> entities) {
        this.baseMapper.deleteEntities(entities);
        return true;
    }

    @Transactional
    public IBIZBOOK finish(IBIZBOOK dto) {
        return dto;
    }

    @Transactional
    public IBIZBOOK init(IBIZBOOK dto) {
        return dto;
    }

    @Transactional
    public IBIZBOOK nextAction(IBIZBOOK dto) {
        return dto;
    }

    @Transactional
    public IBIZBOOK preAction(IBIZBOOK dto) {
        return dto;
    }

    @Transactional
    public IBIZBOOK updatePress(IBIZBOOK dto) {
        return dto;
    }

    public Page<IBIZBOOK> searchDefault(IBIZBOOKSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBIZBOOK> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBIZBOOK>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }
    public List<IBIZBOOK> listDefault(IBIZBOOKSearchContext context) {
        return baseMapper.listDefault(context,context.getSelectCond());
    }





    }