package cn.ibizlab.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;
import lombok.extern.slf4j.Slf4j;

import cn.ibizlab.util.security.SpringContextHolder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.util.Assert;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.sample.domain.IBIZUNIProduct;
import cn.ibizlab.core.sample.filter.IBIZUNIProductSearchContext;
import cn.ibizlab.core.sample.service.IBIZUNIProductService;
import cn.ibizlab.core.sample.mapper.IBIZUNIProductMapper;
import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.security.AuthenticationUser;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.core.conditions.Wrapper;


/**
 * 实体[统一产品（索引主实体）] 服务对象接口实现
 */
@Slf4j
@Service("IBIZUNIProductService")
public class IBIZUNIProductServiceBase extends ServiceImpl<IBIZUNIProductMapper,IBIZUNIProduct> implements IBIZUNIProductService {


    protected int batchSize = 500;

    public IBIZUNIProduct get(IBIZUNIProduct et) {
        IBIZUNIProduct rt = this.baseMapper.selectEntity(et);
        Assert.notNull(rt,"数据不存在,统一产品（索引主实体）:"+et.getIbizuniProductId());
        BeanUtils.copyProperties(rt, et);
        return et;
    }
    
    public List<IBIZUNIProduct> getByEntities(List<IBIZUNIProduct> entities) {
        return this.baseMapper.selectEntities(entities);
    }


    public IBIZUNIProduct getDraft(IBIZUNIProduct et) {
        return et;
    }

    public Integer checkKey(IBIZUNIProduct et) {
        return ( (!ObjectUtils.isEmpty(et.getIbizuniProductId())) && this.count(Wrappers.<IBIZUNIProduct>query().eq("ibizuniproductid", et.getIbizuniProductId()))>0 )?1:0;
    }

    @Override
    @Transactional
    public boolean create(IBIZUNIProduct et) {
        if(!this.retBool(this.baseMapper.insert(et))) 
            return false;
        get(et);
        return true;
    }
    @Transactional
    public boolean createBatch(List<IBIZUNIProduct> list) {
        this.saveBatch(list, batchSize);
        return true;
    }

    @Transactional
    public boolean update(IBIZUNIProduct et) {
        UpdateWrapper qw=et.getUpdateWrapper(true);
        qw.eq("ibizuniproductid", et.getIbizuniProductId());
        if(!update(et, qw)) {
            return false;
        }
        get(et);
        return true;
    }

    @Transactional
    public boolean updateBatch(List<IBIZUNIProduct> list) {
        updateBatchById(list, batchSize);
        return true;
    }

    @Transactional
    public boolean save(IBIZUNIProduct et) {
        if(checkKey(et)>0)
            return getSelf().update(et);
        else
            return getSelf().create(et);
    }

    @Transactional
    public boolean saveBatch(List<IBIZUNIProduct> list) {
        List<IBIZUNIProduct> rt=this.getByEntities(list);
        Set<Serializable> keys=new HashSet<>();
        rt.forEach(et->{
            Serializable key = et.getIbizuniProductId();
            if(!ObjectUtils.isEmpty(key))
                keys.add(key);
        });
        List<IBIZUNIProduct> _create=new ArrayList<>();
        List<IBIZUNIProduct> _update=new ArrayList<>();
        list.forEach(et-> {
            Serializable key = et.getIbizuniProductId();
            if(keys.contains(key))
                _update.add(et);
            else
                _create.add(et);
        });
        List rtList=new ArrayList<>();
        if(_update.size()>0 && (!getSelf().updateBatch(_update)))
            return false;
        if(_create.size()>0 && (!getSelf().createBatch(_create)))
            return false;
        return true;
    }

    @Transactional
    public boolean remove(IBIZUNIProduct et) {
        String key = et.getIbizuniProductId();

        if(!remove(new QueryWrapper<IBIZUNIProduct>()
                .eq("ibizuniproductid", et.getIbizuniProductId())
            )) {
            return false;
        }
        return true ;
    }

    @Transactional
    public boolean removeByEntities(List<IBIZUNIProduct> entities) {
        this.baseMapper.deleteEntities(entities);
        return true;
    }

    public Page<IBIZUNIProduct> searchDefault(IBIZUNIProductSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBIZUNIProduct> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBIZUNIProduct>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }
    public List<IBIZUNIProduct> listDefault(IBIZUNIProductSearchContext context) {
        return baseMapper.listDefault(context,context.getSelectCond());
    }





    }